/* NiuTrans.Tensor - an open-source tensor library
 * Copyright (C) 2018, Natural Language Processing Lab, Northeastern University.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * $Created by: HU Chi (huchinlp@foxmail.com)
 */

#ifndef BLSTM_CRF_MODEL_H_
#define BLSTM_CRF_MODEL_H_

#include <string>

#include "SLTKCRF.h"
#include "SLTKLSTMCell.h"
#include "SLTKDataUtility.h"
#include "SLTKProjection.h"
#include "../../tensor/XGlobal.h"
#include "../../tensor/XTensor.h"

using namespace nts;
using namespace std;
using namespace util;
namespace ner
{

/* bi-lstm-crf model */
struct BiLSTMCRFModel
{
    /* memory pool */
    XMem * mem;

    /* configurations for the model */
    Config *config;

    /* bi-directional LSTM layer */
    BiLSTMLayer *lstm;

    /* projection layer */
    ProjectLayer *projection;

    /* CRF layer */
    CRF *crf;

    /* cache of lstm output and so on */
    XTensor cache[3];

    void GetParams(XList &list) const;

    void StoreModel(const char *fn) const;

    void RestoreModel(const char *fn) const;

    DTYPE Forward(Word2Vec &w2v,
                  const vector<int> &realLengths,
                  const vector<vector<int>> &tagIndices,
                  const vector<vector<wstring>> &sentences);

    void Backward(const vector<int> &realLengths, const vector<vector<int>> &tagIndices);

    void Update(DTYPE lr);

    void ResetCache();

    bool Evaluate(const char *mode, Word2Vec &w2v);

    explicit BiLSTMCRFModel(Config *cfg);

    ~BiLSTMCRFModel();
};

} // namespace ner

#endif // BLSTM_CRF_MODEL_H_