/* NiuTrans.Tensor - an open-source tensor library
 * Copyright (C) 2018, Natural Language Processing Lab, Northeastern University.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * $Created by: HU Chi (huchinlp@foxmail.com)
 */


#ifndef PROJECTION_H_
#define PROJECTION_H_

#include "../../tensor/XTensor.h"

namespace ner
{

/* projection layer */
struct ProjectLayer
{
    /* device id */
    int devID;

    /* memory pool */
    XMem * mem;

    /* train the model or not */
    bool isTraining;

    /* parameters */
    XTensor hiddenW;
    XTensor hiddenB;
    XTensor outputW;
    XTensor outputB;

    XTensor cache[2];

    /* project lstm outputs to the tag space */
    void Forward(XTensor *input, XTensor *output);

    /* back-propagation of this layer */
    void Backward(XTensor *input, XTensor *gradIn, XTensor *gradOut);

    explicit ProjectLayer(int inputDim, int outputDim, int myDevID, XMem *myMem, bool myIsTraining);

    ~ProjectLayer();
};


} // namespace ner

#endif // PROJECTION_H_